// Listing Five
// Sample TWAIN Application Program
// (c) Craig A. Lindley 1993

/*
This program exercises the MYTWAIN.DLL. It is written in Borland's OWL.
It allows the user to select a Source for acquisition and to acquire
an image from the selected Source. Each acquired image is written to
the file c:\scanimg.tif in the root directory of the C drive. Images
are not displayed by this sample application program.
*/

#include <owl.h>
#include "app.h"
#include "twain.hpp"

class TSampleTWAINApp : public TApplication {
  public:
    TSampleTWAINApp(LPSTR AName, HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine, int nCmdShow)
    : TApplication(AName, hInstance, hPrevInstance,
                   lpCmdLine, nCmdShow) {};
    virtual void InitMainWindow();
};


_CLASSDEF(TAppWindow)
class TAppWindow : public TWindow {
  private:
    virtual BOOL CanClose();
    virtual void CMSelectSource(RTMessage Msg)
      = [CM_FIRST + CMID_SELECTSOURCE];
    virtual void CMAcquire(RTMessage Msg)
      = [CM_FIRST + CMID_ACQUIRE];
    virtual void CMHelp(RTMessage Msg)
      = [CM_FIRST + CMID_HELP];

    Twain *TwainClassPtr;         // Pointer the the Twain object

  public:
    TAppWindow(PTWindowsObject AParent, LPSTR ATitle);
   ~TAppWindow();
};

// Window Class Constructor
TAppWindow::TAppWindow(PTWindowsObject AParent, LPSTR ATitle)
             : TWindow(AParent, ATitle) {
 
  AssignMenu("CmdMenu");          // Assign the menu to the window
  TwainClassPtr = new Twain;      // Instantiate the Twain class object
}

// Window Class Destructor
TAppWindow::~TAppWindow() {

  delete TwainClassPtr;           // Delete the Twain class object
}


BOOL TAppWindow::CanClose() {

  return TRUE;                    // Allow the window to close
}

// This function is called when the Select Source menu item is clicked
void TAppWindow::CMSelectSource(RTMessage) {

  // Make a call to the DLL to perform the operation
  TwainClassPtr->SelectSource(HWindow);
}

// This function is called when the Acquire menu item is clicked
void TAppWindow::CMAcquire(RTMessage) {

  // Make a call to the DLL to perform the operation
  TwainClassPtr->ScanImage("c:\\scanimg.tif");
}

// This function is called when the Help menu item is clicked
void TAppWindow::CMHelp(RTMessage) {

  MessageBox(HWindow, "(c) Craig A. Lindley, 1993",
                      "Sample TWAIN Application", MB_OK);
}

void TSampleTWAINApp::InitMainWindow() {

  MainWindow = new TAppWindow(NULL, Name);
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow) {

  TSampleTWAINApp MyApp("Sample TWAIN Application Program",
                         hInstance, hPrevInstance,
                         lpCmdLine, nCmdShow);
  MyApp.nCmdShow = SW_SHOWMAXIMIZED;
  MyApp.Run();
  return MyApp.Status;
}
