/*********************************************************/
/***                  Listing Three                    ***/
/***                   "twain.hpp"                     ***/
/***       This is the interface class for TWAIN       ***/
/***                complient devices.                 ***/
/***                                                   ***/
/***                    written by                     ***/
/***                 Craig A. Lindley                  ***/
/***                                                   ***/
/***      Revision: 1.0   Last Update: 12/11/93        ***/
/*********************************************************/

// See the file twain.cpp for the revision history

// Check to see if this file already included
#ifndef TWAIN_HPP
#define TWAIN_HPP

#include "dc.h"
#include "containr.hpp"

/*
The following is required when using a C++ class as an interface
for a DLL. It is required because this file will be included in
both the application compilation and the DLL compilation. Remember
that all members (functions and data) are far!
*/
#ifdef __DLL__
#  define EXPORT _export
#else
#  define EXPORT huge
#endif

#define WINDIRPATHSIZE  160
#define WM_XFERDONE     WM_USER + 100

class EXPORT Twain : public Containr {
  private:
    // Private Data
    LPSTR        lpFileName;
    DC_IDENTITY  AppIdentity;
    DC_IDENTITY  SourceIdentity;
    DSMENTRYPROC lpDSM_Entry;
    BOOL         DSMOpen;
    BOOL         DSOpen;
    BOOL         DSEnabled;
    HINSTANCE    hDSMDLL;
    HINSTANCE    hDLLInstance;
    HWND         hBackgroundWindow;
    HWND         hAppWindow;
    DC_UINT16    XferMechanism;
    BOOL         ScanResults;

    // Private Functions
    void TransferImage(HWND hWindow);
    BOOL GetCompleteImage (HWND hWindow);
    BOOL ProcessDCMessage(LPMSG lpMsg, HWND hWindow);
    void ExplainError(HWND hWnd, LPSTR lpText, int MsgNumber);
    BOOL RestrictToRGB(void);
    BOOL SetXferMechanism(DC_UINT16 Mechanism);
    BOOL SetupFileTransfer(LPSTR FileName);
    BOOL SetResolution(int DPI);
    BOOL OpenDSM(HWND hWindow);
    BOOL CloseDSM(void);
    BOOL OpenDS(void);
    BOOL CloseDS(void);
    BOOL EnableDS(void);
    BOOL DisableDS(void);
    BOOL SelectDS(void);


  public:
    FAR  Twain(void);             // Class constructor/destructor
    FAR ~Twain(void);

    BOOL FAR SelectSource(HWND hWindow);
    BOOL FAR ScanImage(LPSTR FileName);

    static long CALLBACK _export
     BackDropWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

};

#endif
